import knex from 'knex';
import type { MatchResponse, PlayerResponse } from './pubg-api.js';
import dayjs from 'dayjs';

interface Player {
    id: number
    data: string
    created_at: number
}

interface Match {
    match_id: string
    data: string
    created_at: number
}

export async function Database() {
    const db = knex({
        client: 'better-sqlite3',
        connection: {
            filename: 'DOCKER' in process.env ? '/data/pubg.sqlite' : './pubg.sqlite'
        },
        useNullAsDefault: true
    });

    db.on('start', builder => {
        console.log(builder.toQuery());
    })

    if (!await db.schema.hasTable('player')) {
        await db.schema.createTable('player', table => {
            table.increments('id').primary();
            table.json('data');
            table.integer('created_at').notNullable();
        });
    }

    if (!await db.schema.hasTable('match')) {
        await db.schema.createTable('match', table => {
            table.string('match_id').primary();
            table.json('data');
            table.integer('created_at').notNullable();
        });
    }

    return new class Database {
        async getCachedPlayer() {
            const sec = dayjs().add(-10, 'seconds').unix();

            const r = await db<Player>('player').where('created_at', '>', sec).first();
            console.log('cached player response', r);
            if (r) return JSON.parse(r.data) as PlayerResponse;
        }

        async savePlayer(player: PlayerResponse) {
            await db<Player>('player').insert({
                data: JSON.stringify(player),
                created_at: dayjs().unix()
            });
            console.log('saved cached player response', player);
        }

        async getMatch(match_id: string) {
            const match = await db<Match>('match').where('match_id', match_id).first();
            if (match) return JSON.parse(match.data) as MatchResponse;
        }

        async saveMatch(match: MatchResponse) {
            const match_id = match.data.id;
            await db<Match>('match').insert({
                match_id,
                data: JSON.stringify(match),
                created_at: dayjs().unix()
            }).onConflict('match_id').ignore();
        }
    }
}
