import * as changeCase from 'change-case';

const API_KEY = process.env.API_KEY;

function getAcceptHeaders() {
    return {
        'Accept': 'application/vnd.api+json'
    }
}

function getAuthHeaders() {
    return {
        'Authorization': `Bearer ${API_KEY}`,
        ...getAcceptHeaders()
    }
}

export interface PlayerResponse {
    data: {
        id: string
        attributes: {
            name: string
        }
        relationships: {
            matches: Relationship
        }
    }[]
}

interface Relationship {
    data: {
        id: string
        type: string
    }[]
}

export interface MatchResponse {
    data: {
        id: string
        attributes: {
            mapName: string
            duration: number
            gameMode: string
            createdAt: string
        },
        relationships: {
            rosters: Relationship
            assets: Relationship
        }
    },
    included: (Participant | Roster | Asset)[]
}

export interface Participant {
    type: 'participant'
    id: string
    attributes: {
        stats: {
            playerId: string
            name: string
            winPlace: number
        }
    }
}
export interface Roster {
    type: 'roster'
    id: string
    attributes: {
        stats: {
            rank: number
            teamId: number
        }
        won: string
    }
    relationships: {
        participants: Relationship
    }
}
export interface Asset {
    type: 'asset'
    id: string
    attributes: {
        createdAt: string
        URL: string
    }
}

const mapLookup: Record<string, string> = {
    "Baltic_Main": "Erangel",
    "Chimera_Main": "Paramo",
    "Desert_Main": "Miramar",
    "DihorOtok_Main": "Vikendi",
    "Erangel_Main": "Erangel",
    "Heaven_Main": "Haven",
    "Kiki_Main": "Deston",
    "Range_Main": "Camp Jackal",
    "Savage_Main": "Sanhok",
    "Summerland_Main": "Karakin",
    "Tiger_Main": "Taego",
    "Neon_Main": "Rondo"
}

export class PubgApi {
    async getPlayers(playerNames: string[]) {
        const result = await fetch('https://api.pubg.com/shards/steam/players?filter[playerNames]=' + playerNames.join(','), {
            headers: getAuthHeaders()
        });
        if (!result.ok) throw new Error('cant fetch players');
        const json = await result.json() as PlayerResponse;
        return json;
    }

    async getMatch(id: string) {
        const result = await fetch('https://api.pubg.com/shards/steam/matches/' + id, {
            headers: getAcceptHeaders()
        });
        if (!result.ok) throw new Error('cant fetch');
        const json = await result.json() as MatchResponse;
        return json;
    }

    static mapName(name: string) {
        return mapLookup[name];
    }

    static modeName(name: string) {
        console.log(name);
        let [team, per] = name.split('-');
        if(!per) per = 'tpp';
        return per.toUpperCase() + ' ' + changeCase.capitalCase(team);
    }
}
